CREATE SEQUENCE seq_global_disciplines
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

select setval('seq_global_disciplines', 1, true);

CREATE TABLE global_disciplines
(
    id           integer DEFAULT nextval('seq_global_disciplines'::regclass) NOT NULL,
    name         character varying(100)                          NOT NULL,
    disciplineid integer                                        NOT NULL,
    semesterid integer                                          NOT NULL
);

ALTER TABLE ONLY global_disciplines
    ADD CONSTRAINT global_name_key UNIQUE (name);

ALTER TABLE ONLY global_disciplines
    ADD CONSTRAINT global_disciplines_name_key UNIQUE (name, semesterid, disciplineid);

ALTER TABLE ONLY global_disciplines
    ADD CONSTRAINT global_disciplines_ibfk_1 FOREIGN KEY (disciplineid) REFERENCES disciplines (id);

ALTER TABLE disciplines
    ADD COLUMN isglobal bool;

ALTER TABLE disciplines
    ALTER COLUMN isglobal set default FALSE;

UPDATE disciplines SET isglobal = FALSE;


-- ищет межфакультетские дисциплины по семестру, ведущему факультету, предмету и типу
-- межфакультетскими считает все дисциплины, которые не связаны с учебными планами
DROP FUNCTION IF EXISTS public.discipline_find_global(psemesterid integer, psubjectid integer, pexamtype exam_credit_grading_credit, pfacultyid integer, pdisciplinename character varying);
CREATE OR REPLACE FUNCTION public.discipline_find_global(psemesterid integer, psubjectid integer, pexamtype exam_credit_grading_credit, pfacultyid integer, pdisciplinename character varying)
    RETURNS SETOF integer
    LANGUAGE sql
AS $function$
    --select public.discipline_find_by_subject(
--	:psemesterid,	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--	:psubjectid,	-- put the psubjectid parameter value instead of 'psubjectid' (int4)
--	:pexamtype 	-- put the pexamtype parameter value instead of 'pexamtype' (exam_credit_grading_credit)
--);
SELECT disciplines.ID
FROM disciplines
JOIN global_disciplines ON global_disciplines.disciplineid = disciplines.id
WHERE disciplines.SemesterID = pSemesterID AND
        --disciplines.SubjectID = pSubjectID AND
        --disciplines.ExamType = pExamType AND
        --disciplines.FacultyID = pfacultyid AND
        -- NOT EXISTS (SELECT disciplines_study_plans.id from disciplines_study_plans WHERE disciplines_study_plans.DisciplineID = disciplines.ID) AND
        disciplines.isglobal = TRUE AND
        global_disciplines.name = pdisciplinename
LIMIT 1;
$function$;

DROP FUNCTION IF EXISTS public.discipline_bindglobal(pdisciplineid integer, psemester integer, pdisciplinename character varying);
CREATE OR REPLACE FUNCTION public.discipline_bindglobal(pdisciplineid integer, psemester integer, pdisciplinename character varying)
    RETURNS integer
    LANGUAGE plpgsql
AS $function$
declare vID int;
begin
INSERT INTO global_disciplines (disciplineid, semesterid, name) VALUES(pdisciplineid, psemester, pdisciplinename)  returning id into vID;
UPDATE disciplines SET isglobal=TRUE WHERE disciplines.id = pdisciplineid;
RETURN vID;
END;
$function$;

DROP FUNCTION IF EXISTS public.internalbindstudent(pdisciplineid integer, precordbookid integer);
CREATE OR REPLACE FUNCTION public.internalbindstudent(pdisciplineid integer, precordbookid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
BEGIN
    -- bind  student
    INSERT INTO disciplines_students (DisciplineID, recordbookid, "type") VALUES (pDisciplineID, precordbookid, 'attach')
    ON conflict ON CONSTRAINT disciplines_students_recordbookid_disciplineid_key
        DO UPDATE set "type" = 'attach';
   return 1;
   EXCEPTION 
    when others then RETURN -1;
END
$function$;

DROP FUNCTION IF EXISTS public.internalunbindstudent(pdisciplineid integer, precordbookid integer);
CREATE OR REPLACE FUNCTION public.internalunbindstudent(pdisciplineid integer, precordbookid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
BEGIN
    -- unbind  student
    update disciplines_students 
    set "type" = 'detach'
    where DisciplineID=pdisciplineid and recordbookid=precordbookid;
    return 1;
   EXCEPTION 
    when others then RETURN -1;
END
$function$;

DROP VIEW IF EXISTS public.view_disciplines;
CREATE OR REPLACE VIEW public.view_disciplines
AS SELECT disciplines.id AS disciplineid,
          disciplines.authorid,
          disciplines.examtype,
          disciplines.lecturecount,
          disciplines.practicecount,
          disciplines.labcount,
          disciplines.semesterid,
          disciplines.islocked,
          disciplines.milestone,
          disciplines.subtype,
          disciplines.compounddiscid,
          disciplines.maxrate,
          disciplines.currate,
          disciplines.isinactive,
          disciplines.isglobal,
          grades.id AS gradeid,
          grades.num AS gradenum,
          grades.degree,
          subjects.id AS subjectid,
          subjects.name AS subjectname,
          subjects.abbr AS subjectabbr,
          faculties.id AS facultyid,
          faculties.name AS facultyname,
          faculties.abbr AS facultyabbr,
          compound_disciplines.name AS compounddiscname,
          global_disciplines.name AS globalname
   FROM disciplines
            JOIN subjects ON subjects.id = disciplines.subjectid
            JOIN faculties ON faculties.id = disciplines.facultyid
            LEFT JOIN grades ON grades.id = disciplines.gradeid
            LEFT JOIN compound_disciplines ON compound_disciplines.id = disciplines.compounddiscid
            LEFT JOIN global_disciplines ON global_disciplines.disciplineid = disciplines.id;

DROP FUNCTION IF EXISTS public.discipline_getinfo(pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.discipline_getinfo(pdisciplineid integer)
    RETURNS TABLE("ID" integer, "AuthorID" integer, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "Type" exam_credit_grading_credit, "Lectures" integer, "Practice" integer, "Labs" integer, "SemesterID" integer, "SubjectID" integer, "SubjectName" character varying, "SubjectAbbr" character varying, "FacultyID" integer, "FacultyName" character varying, "IsLocked" integer, "Milestone" integer, "Subtype" scientific_disciplinary_coursework, "CompoundDiscID" integer, "IsMapCreated" boolean, "IsGlobal" boolean, "IsBonus" boolean, "semesterNum" integer, "semesterYear" integer, "IsInactive" bool, "GlobalName" character varying)
    LANGUAGE plpgsql
AS $function$
DECLARE vIsBonus BOOLEAN;
begin
    --	select * from public.discipline_getinfo(
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);
    vIsBonus := EXISTS(
            SELECT * FROM modules
            WHERE   modules.DisciplineID = pDisciplineID AND
                    modules.Type = 'bonus'
            LIMIT 1
        );
    return query
        SELECT
            view_disciplines.DisciplineID AS "ID",
            view_disciplines.AuthorID as "AuthorID",
            view_disciplines.GradeID as "GradeID",
            view_disciplines.GradeNum as "GradeNum",
            view_disciplines."degree" as "Degree",
            view_disciplines.ExamType       AS "Type",
            view_disciplines.LectureCount   AS "Lectures",
            view_disciplines.PracticeCount  AS "Practice",
            view_disciplines.LabCount       AS "Labs",
            view_disciplines.SemesterID as "SemesterID",
            view_disciplines.SubjectID as "SubjectID",
            view_disciplines.SubjectName as "SubjectName",
            view_disciplines.SubjectAbbr as "SubjectAbbr",
            view_disciplines.FacultyID as "FacultyID",
            view_disciplines.FacultyName as "FacultyName",
            view_disciplines.IsLocked as "IsLocked",
            view_disciplines.Milestone as "Milestone",
            view_disciplines.Subtype as "Subtype",
            view_disciplines.CompoundDiscID as "CompoundDiscID",
            (view_disciplines.MaxRate = 100) AS "IsMapCreated",
            --NOT EXISTS (SELECT disciplines_study_plans.id from disciplines_study_plans WHERE disciplines_study_plans.DisciplineID = view_disciplines.DisciplineID) AS "IsGlobal",
            view_disciplines.IsGlobal AS "IsGlobal",
            vIsBonus AS "IsBonus",
            semesters.Num AS "semesterNum", -- TODO: Camelize
            semesters.Year AS "semesterYear",
            view_disciplines.IsInactive as "IsInactive",
            view_disciplines.GlobalName AS "GlobalName"
        FROM view_disciplines
                 INNER JOIN semesters ON semesters.ID = view_disciplines.SemesterID
        WHERE view_disciplines.DisciplineID = pDisciplineID
        LIMIT 1;
END ;
$function$;

DROP FUNCTION IF EXISTS public.getdisciplinesforteacher(pteacherid integer, psemesterid integer);
CREATE OR REPLACE FUNCTION public.getdisciplinesforteacher(pteacherid integer, psemesterid integer)
    RETURNS TABLE("ID" integer, "Type" exam_credit_grading_credit, "SubType" scientific_disciplinary_coursework, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "GroupID" integer, "GroupNum" integer, "GroupName" character varying, "SubjectID" integer, "SubjectName" character varying, "AuthorID" integer, "IsLocked" integer, "IsMapCreated" boolean, "IsGlobal" boolean, "GlobalName" character varying)
    LANGUAGE sql
AS $function$
    --select public.getdisciplinesforteacher(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--);
SELECT DISTINCT view_disciplines.DisciplineID AS "ID",
                view_disciplines.ExamType AS "Type",
                view_disciplines.Subtype as "SubType",
                view_disciplines.GradeID as "GradeID",
                view_disciplines.GradeNum as "GradeNum",
                view_disciplines."degree" as "Degree",
                view_groups.GroupID as "GroupID",
                view_groups.GroupNum as "GroupNum",
                view_groups.GroupName as "GroupName",
                view_disciplines.SubjectID as "SubjectID",
                view_disciplines.SubjectName as "SubjectName",
                view_disciplines.AuthorID as "AuthorID",
                view_disciplines.IsLocked AS "IsLocked", -- bodging, db schema remembered lowerCase
                (view_disciplines.MaxRate = 100)    AS "IsMapCreated",
                --NOT EXISTS (SELECT disciplines_study_plans.id from disciplines_study_plans WHERE disciplines_study_plans.DisciplineID = view_disciplines.DisciplineID) AS "IsGlobal"
                view_disciplines.IsGlobal AS "IsGlobal",
                view_disciplines.GlobalName AS "GlobalName"
FROM disciplines_teachers
         INNER JOIN view_disciplines ON disciplines_teachers.DisciplineID = view_disciplines.DisciplineID
         INNER JOIN semesters ON semesters.ID = view_disciplines.SemesterID
         LEFT JOIN disciplines_groups ON disciplines_groups.DisciplineID = disciplines_teachers.DisciplineID
         LEFT JOIN view_groups ON view_groups.GroupID = disciplines_groups.GroupID AND view_groups.Year = semesters.Year
WHERE disciplines_teachers.TeacherID = pTeacherID AND view_disciplines.SemesterID = pSemesterID AND view_disciplines.IsInactive = FALSE
ORDER BY    view_disciplines.GradeID ASC,
            view_disciplines.SubjectName ASC,
            view_disciplines.DisciplineID ASC,
            view_groups.GroupNum ASC;
$function$;

DROP FUNCTION IF EXISTS public.getfinalforminfoglobal(pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.getfinalforminfoglobal(pdisciplineid integer)
    RETURNS TABLE("FacultyID" integer, "FacultyName" character varying, "FacultyAbbr" character varying, "ExamType" exam_credit_grading_credit, "SubjectID" integer, "SubjectName" character varying, "SubjectAbbr" character varying, "AuthorID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "JobPosition" character varying, "Year" integer, "SemesterNum" integer)
    LANGUAGE sql
AS $function$
SELECT  faculties.ID AS "FacultyID",
        faculties.Name AS "FacultyName",
        faculties.Abbr AS "FacultyAbbr",
        disciplines.ExamType AS "ExamType",
        subjects.ID AS "SubjectID",
        subjects.Name AS "SubjectName",
        subjects.Abbr AS "SubjectAbbr",
        teachers.ID AS "AuthorID",
        accounts.LastName AS "LastName",
        accounts.FirstName AS "FirstName",
        accounts.SecondName AS "SecondName",
        job_positions.Name AS "JobPosition",
        semesters."year" AS "Year",
        semesters.Num AS "SemesterNum"
FROM disciplines
         INNER JOIN subjects ON disciplines.SubjectID = subjects.ID
         INNER JOIN teachers ON teachers.ID = disciplines.AuthorID
         INNER JOIN accounts ON teachers.AccountID = accounts.ID
         INNER JOIN job_positions ON job_positions.ID = teachers.JobPositionID
         INNER JOIN semesters ON disciplines.SemesterID = semesters.ID
         INNER JOIN faculties ON faculties.ID = disciplines.FacultyID
WHERE disciplines.ID = pdisciplineid
LIMIT 1;
$function$;

DROP FUNCTION IF EXISTS public.getratesforstudentsbystage(pdisciplineid integer, pmilestone integer);
CREATE OR REPLACE FUNCTION public.getratesforstudentsbystage(pdisciplineid integer, pmilestone integer)
    RETURNS TABLE("StudentID" integer, "Semester" integer, "Bonus" integer, "Extra" integer, "PreviousExtra" integer, "Exam" integer, "PreviousExam" integer, "Option" absence_pass, "AutoPassed" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying)
    LANGUAGE plpgsql
AS $function$

DECLARE vSemesterID INT DEFAULT -1;
        vInGeneralGroup BOOL DEFAULT FALSE;
BEGIN
    vSemesterID := GetDisciplineProperty(pDisciplineID, 'semester');

    DROP TABLE IF EXISTS tStudents;
    CREATE TEMPORARY TABLE tStudents AS (
        SELECT record_books.ID, record_books.StudentID
        FROM record_books
                 LEFT
                     JOIN disciplines_students on disciplines_students.DisciplineID = pDisciplineid and
                                                  disciplines_students.RecordBookID = record_books.ID
                 left join view_disciplines_recordbooks on disciplines_students.recordbookid = view_disciplines_recordbooks.recordbookid
        WHERE   COALESCE(disciplines_students.Type, 'attach') = 'attach' and
                view_disciplines_recordbooks.disciplineid = pDisciplineid
    );

    return query
        SELECT  tRes.*,
                accounts.LastName as "LastName",
                accounts.FirstName as "FirstName",
                accounts.SecondName as "SecondName"
        FROM (
                 SELECT  tStudents.StudentID as "StudentID",
                         cast (SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'regular',1,0)) as integer) AS "Semester",
                         cast (SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'bonus',1,0)) as integer) AS "Bonus",
                         cast (SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'extra',1,0)*iif_sql(tMap.SubmoduleOrderNum < pMilestone,1,0)) as integer) AS "Extra",
                         cast (SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'extra',1,0)*iif_sql(tMap.SubmoduleOrderNum < pMilestone - 1,1,0)) as integer) AS "PreviousExtra",
                         cast (SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'exam',1,0)*iif_sql(tMap.SubmoduleOrderNum = pMilestone,1,0)) as integer) AS "Exam",
                         MAX(tRate.Rate*iif_sql(tMap.ModuleType = 'exam',1,0)*iif_sql(tMap.SubmoduleOrderNum < pMilestone,1,0)) AS "PreviousExam",
                         MAX(CASE WHEN tMap.SubmoduleOrderNum = pMilestone THEN  exam_period_options."type" ELSE  NULL END) As "Option",
                         MAX(CASE WHEN exam_period_options."type" = 'pass' THEN  1 ELSE  0 END) As "AutoPassed"
                 FROM tStudents
                          LEFT JOIN view_roadmap AS tMap   ON  tMap.DisciplineID = pDisciplineID
                          LEFT JOIN rating_table AS tRate  ON  tRate.RecordbookID = tStudents.ID AND
                                                               tRate.SubmoduleID = tMap.SubmoduleID
                          LEFT JOIN exam_period_options ON  exam_period_options.submoduleID = tMap.SubmoduleID AND
                                                            exam_period_options.RecordbookID = tStudents.ID
                 GROUP BY tStudents.StudentID
             ) tRes
                 INNER JOIN students ON students.ID = tRes."StudentID"
                 INNER JOIN accounts ON students.AccountID = accounts.ID
        ORDER BY    accounts.LastName ASC,
                    accounts.FirstName ASC,
                    accounts.SecondName ASC;
END
$function$;

DROP FUNCTION IF EXISTS public.getdisciplinesforgroup(pgroupid integer, psemesterid integer);
CREATE OR REPLACE FUNCTION public.getdisciplinesforgroup(pgroupid integer, psemesterid integer)
    RETURNS TABLE("ID" integer, "SubjectName" character varying, "Subtype" scientific_disciplinary_coursework, "Type" exam_credit_grading_credit, "CompoundDiscID" integer, "CompoundDiscName" character varying, "CurRate" integer, "MaxRate" integer)
    LANGUAGE plpgsql
AS $function$
BEGIN
    return query
        SELECT DISTINCT view_disciplines.DisciplineID AS "ID",
                        view_disciplines.SubjectName as "SubjectName",
                        view_disciplines.Subtype as "Subtype",
                        view_disciplines.ExamType  AS "Type",
                        view_disciplines.CompoundDiscID as "CompoundDiscID",
                        view_disciplines.CompoundDiscName as "CompoundDiscName",
                        view_disciplines.CurRate as "CurRate",
                        view_disciplines.MaxRate as "MaxRate"
        FROM students_groups
                 LEFT JOIN view_disciplines_recordbooks ON students_groups.RecordBookID = view_disciplines_recordbooks.RecordBookID
                 INNER JOIN view_disciplines ON view_disciplines_recordbooks.DisciplineID = view_disciplines.DisciplineID
        WHERE   students_groups.GroupID = pGroupID AND
                students_groups.SemesterID = pSemesterID AND
                view_disciplines.SemesterID = pSemesterID AND
                view_disciplines.IsInactive = False AND
                view_disciplines.IsGlobal = False AND
                students_groups.State <= 'outlet' AND
          -- view_disciplines_recordbooks.Type may be attach or NULL
                COALESCE(view_disciplines_recordbooks.Type, 'attach') = 'attach'
        ORDER BY view_disciplines.ExamType DESC,
                 view_disciplines.SubjectName ASC;
end;
$function$;