-- функция используется при поиске группы для выгрузки ведомости в 1С
-- добавлено сравнение формы обучения, чтобы ведомости для ВО и ЗО выгружались правильно
drop function if exists public.findgroup(pgradeid integer, pgroupnum integer, pfacultyid integer, pformid integer);
CREATE OR REPLACE FUNCTION public.findgroup(pgradeid integer, pgroupnum integer, pfacultyid integer, pformid integer)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
begin

  RETURN COALESCE((
                    SELECT study_groups.ID FROM study_groups
                    WHERE   study_groups.GradeID = pGradeID
                            AND study_groups.GroupNum = pGroupNum
                            AND study_groups.FacultyID = pFacultyID
                            AND study_groups.formid = pFormID
                    LIMIT 1
                  ), -1);
END
$function$;

DROP FUNCTION IF EXISTS public.GetAccountExtIdByInila(pinila character varying);
CREATE OR REPLACE FUNCTION public.GetAccountExtIdByInila(pinila character varying)
  RETURNS character varying
LANGUAGE sql
AS $function$
SELECT accounts.externalid FROM accounts
WHERE inila = pinila;

$function$;

DROP FUNCTION IF EXISTS public.signstudentbyopenid(pglobalkey character varying);
CREATE OR REPLACE FUNCTION public.signstudentbyopenid(pglobalkey character varying)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
DECLARE vAccountID INT DEFAULT null;
begin
  --check account existence
  SELECT accounts.ID INTO vAccountID FROM accounts
  WHERE concat('st-', accounts.ExternalID) = pGlobalKey
        and accounts.userroleid = 1
  LIMIT 1;
  IF vAccountID is null  THEN
    RETURN -1;
  END IF;

  INSERT INTO logs_signin (AccountID) VALUES (vAccountID);
  RETURN vAccountID;
END
$function$;

ALTER TABLE lessons ADD COLUMN lesson_name CHARACTER VARYING DEFAULT '';

DROP FUNCTION IF EXISTS public.get_lessons_for_discipline(pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.get_lessons_for_discipline(pdisciplineid integer)
  RETURNS TABLE("ID" integer, "LessonDate" date, "LessonType" integer, "LessonName" character varying)
LANGUAGE sql
AS $function$
SELECT lessons.id AS "ID",
       lessons.lesson_date AS "LessonDate",
       lessons.lesson_type AS "LessonType",
       lessons.lesson_name AS "LessonName"
FROM lessons
WHERE lessons.discipline_id = pdisciplineid
      AND lessons.lesson_deleted = 0
order by lessons.lesson_date;
$function$;

CREATE OR REPLACE FUNCTION public.set_lesson_name(plessonid integer, pname character varying)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
declare vRowUpd int;
begin
  update lessons set lesson_name = pname
  where lessons.id = plessonid;

  get diagnostics vRowUpd = ROW_COUNT;
  RETURN vRowUpd - 1;
end
$function$;

--вспомогательная функция для getstudentsforrating
CREATE OR REPLACE FUNCTION public.discipline_getstudentstemp(pdisciplineid integer)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
begin
  --select grade.Discipline_GetStudentsTemp(
  ---	:pDisciplineID 	-- put the pDisciplineID parameter value instead of 'pDisciplineID' (INT)
  --);
  CREATE TEMPORARY TABLE tDisciplineStudents AS (
    SELECT  record_books.StudentID,record_books.id,
      COALESCE(view_disciplines_recordbooks."type") AS "Type" ---?????
    FROM view_disciplines_recordbooks
      INNER JOIN record_books ON view_disciplines_recordbooks.RecordBookID = record_books.ID
    WHERE view_disciplines_recordbooks.DisciplineID = pDisciplineID
    GROUP BY record_books.StudentID,record_books.id,COALESCE(view_disciplines_recordbooks."type")
  );
  RETURN 0;
end;
$function$;

CREATE OR REPLACE FUNCTION public.getstudentsforrating(pdisciplineid integer)
  RETURNS TABLE("ID" integer, "RecordBookID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "GroupID" integer, "GroupNum" integer, "IsAttached" boolean)
LANGUAGE plpgsql
AS $function$

DECLARE vSemesterID INT DEFAULT -1; vRes INT DEFAULT -1;
BEGIN
  vSemesterID := GetDisciplineProperty(pDisciplineID, 'semester');

  DROP TABLE IF EXISTS tDisciplineStudents;
  vRes := Discipline_GetStudentsTemp(pDisciplineID);
  return query
  SELECT  view_students.StudentID 		AS "ID",
          view_students.RecordBookID		as "RecordBookID",
          view_students.LastName			as "LastName",
          view_students.FirstName			as "FirstName",
          view_students.SecondName		as "SecondName",
          view_students.GradeID			as "GradeID",
          view_students.GradeNum			as "GradeNum",
          view_students."degree"			as "Degree",
          view_students.GroupID			as "GroupID",
          view_students.GroupNum			as "GroupNum",
          ( tDisciplineStudents."Type" IS NOT NULL ) AS "IsAttached"
  FROM tDisciplineStudents
    INNER JOIN view_students ON   view_students.StudentID = tDisciplineStudents.StudentID AND
                                  view_students.SemesterID = vSemesterID
                                  and view_students.RecordBookID = tDisciplineStudents.id
                                  and view_students.recordbookid in (select recordbookid from students_groups where state <= 'outlet' and semesterid=vsemesterid)
  WHERE  tDisciplineStudents."Type" is null or ( tDisciplineStudents."Type" !='detach')
  ORDER BY    view_students.GradeID ASC,
    view_students.GroupNum ASC,
    view_students.LastName ASC,
    view_students.FirstName ASC;
END;
$function$;

CREATE OR REPLACE FUNCTION public.getstudents(pfacultyid integer, pgradeid integer, pgroupid integer, psemesterid integer, pword1 character varying, pword2 character varying, pword3 character varying)
  RETURNS TABLE("ID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "AccountID" integer, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "GroupID" integer, "GroupNum" integer)
LANGUAGE plpgsql
AS $function$
DECLARE FullName VARCHAR;
begin
  return query
  SELECT  view_students.StudentID AS "ID",
          view_students.LastName AS "LastName",
          view_students.FirstName AS "FirstName",
          view_students.SecondName AS "SecondName",
          view_students.AccountID AS "AccountID",
          view_students.GradeID AS "GradeID",
          view_students.GradeNum AS "GradeNum",
          view_students.Degree AS "Degree",
          view_students.GroupID AS "GroupID",
          view_students.GroupNum AS "GroupNum"
  FROM view_students
  where
    recordbookid in (select recordbookid from students_groups where semesterid=psemesterid and state<='outlet') and
    CASE WHEN pSemesterID != 0 THEN view_students.SemesterID = pSemesterID ELSE
      TRUE
    END AND
    CASE WHEN pFacultyID != 0 THEN view_students.FacultyID = pFacultyID ELSE TRUE END AND
    CASE WHEN pGradeID != 0 THEN view_students.GradeID = pGradeID ELSE TRUE END AND
    CASE WHEN pGroupID != 0 THEN view_students.GroupID = pGroupID ELSE TRUE END AND
    CASE WHEN pWord1 != '' AND FullName != '' THEN

      CONCAT(view_students.LastName, ' ', view_students.FirstName, ' ', view_students.SecondName) LIKE CONCAT('%', pWord1, '%') AND
      CONCAT(view_students.LastName, ' ', view_students.FirstName, ' ', view_students.SecondName) LIKE CONCAT('%', pWord2, '%') AND
      CONCAT(view_students.LastName, ' ', view_students.FirstName, ' ', view_students.SecondName) LIKE CONCAT('%', pWord3, '%')
    ELSE TRUE end
  order by "LastName","FirstName","SecondName";
END
$function$;

CREATE OR REPLACE FUNCTION public.getratesforgroupbystage(pdisciplineid integer, pgroupid integer, pmilestone integer)
  RETURNS TABLE("StudentID" integer, "Semester" integer, "Bonus" integer, "Extra" integer, "PreviousExtra" integer, "Exam" integer, "PreviousExam" integer, "Option" absence_pass, "AutoPassed" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying)
LANGUAGE plpgsql
AS $function$

DECLARE vSemesterID INT DEFAULT -1; vGroupID INT DEFAULT -1;
        vInGeneralGroup BOOL DEFAULT FALSE;
BEGIN
  vSemesterID := GetDisciplineProperty(pDisciplineID, 'semester');
  /*DROP TABLE IF EXISTS tStudents;
  CREATE TEMPORARY TABLE tStudents (
      StudentID INT NOT NULL
  );*/

  -- check that group attached to discipline. Otherwise vGroupID = -1;
  vInGeneralGroup = EXISTS(
      SELECT * FROM disciplines_groups
      WHERE disciplines_groups.DisciplineID = pDisciplineID AND
            disciplines_groups.GroupID = pGroupID
      LIMIT 1
  );


  /*  DROP TABLE IF EXISTS tStudents;
    CREATE TEMPORARY TABLE tStudents AS (
      SELECT record_books.ID, record_books.StudentID
      FROM students_groups
        LEFT JOIN record_books ON record_books.ID = students_groups.RecordBookID
        left JOIN disciplines_students ON    disciplines_students.DisciplineID = pDisciplineID AND
                                            disciplines_students.RecordBookID = record_books.ID
      WHERE   students_groups.SemesterID = vSemesterID AND
              students_groups.State <= 'outlet' AND --actual students
              students_groups.GroupID = pGroupID and
              COALESCE(disciplines_students.Type, 'attach') = 'attach'


              --CASE WHEN vInGeneralGroup THEN
               -- NOT disciplines_students.Type <=> 'detach' # not detached
             -- ELSE
               -- COALESCE(disciplines_students.Type, 'attach') = 'attach' # is attached
             -- END
    );*/
  DROP TABLE IF EXISTS tStudents;
  CREATE TEMPORARY TABLE tStudents AS (
    SELECT record_books.ID, record_books.StudentID
    FROM students_groups
      --left
      JOIN record_books ON record_books.ID = students_groups.RecordBookID
      LEFT
      JOIN disciplines_students on disciplines_students.DisciplineID = pDisciplineid and
                                   disciplines_students.RecordBookID = record_books.ID
      left join view_disciplines_recordbooks on students_groups.recordbookid = view_disciplines_recordbooks.recordbookid
    WHERE   students_groups.SemesterID = vSemesterID AND
            students_groups.State <= 'outlet' AND
            students_groups.GroupID = pgroupid and
            COALESCE(disciplines_students.Type, 'attach') = 'attach' and
            view_disciplines_recordbooks.disciplineid = pDisciplineid
  );

  return query
  SELECT  tRes.*,
    accounts.LastName as "LastName",
    accounts.FirstName as "FirstName",
    accounts.SecondName as "SecondName"
  FROM (
         SELECT  tStudents.StudentID as "StudentID",
                 cast (SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'regular',1,0)) as integer) AS "Semester",
                 cast (SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'bonus',1,0)) as integer) AS "Bonus",
                 cast (SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'extra',1,0)*iif_sql(tMap.SubmoduleOrderNum < pMilestone,1,0)) as integer) AS "Extra",
                 cast (SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'extra',1,0)*iif_sql(tMap.SubmoduleOrderNum < pMilestone - 1,1,0)) as integer) AS "PreviousExtra",
                 cast (SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'exam',1,0)*iif_sql(tMap.SubmoduleOrderNum = pMilestone,1,0)) as integer) AS "Exam",
                 MAX(tRate.Rate*iif_sql(tMap.ModuleType = 'exam',1,0)*iif_sql(tMap.SubmoduleOrderNum < pMilestone,1,0)) AS "PreviousExam",
                 MAX(CASE WHEN tMap.SubmoduleOrderNum = pMilestone THEN  exam_period_options."type" ELSE  NULL END) As "Option",
                 MAX(CASE WHEN exam_period_options."type" = 'pass' THEN  1 ELSE  0 END) As "AutoPassed"
         FROM tStudents
           LEFT JOIN view_roadmap AS tMap   ON  tMap.DisciplineID = pDisciplineID
           LEFT JOIN rating_table AS tRate  ON  tRate.RecordbookID = tStudents.ID AND
                                                tRate.SubmoduleID = tMap.SubmoduleID
           LEFT JOIN exam_period_options ON  exam_period_options.submoduleID = tMap.SubmoduleID AND
                                             exam_period_options.RecordbookID = tStudents.ID
         GROUP BY tStudents.StudentID
       ) tRes
    INNER JOIN students ON students.ID = tRes."StudentID"
    INNER JOIN accounts ON students.AccountID = accounts.ID
  ORDER BY    accounts.LastName ASC,
    accounts.FirstName ASC,
    accounts.SecondName ASC;
END
$function$;

CREATE OR REPLACE FUNCTION public.getratesforgroupall(pgroupid integer, psemesterid integer)
  RETURNS TABLE("StudentID" integer, "DisciplineID" integer, "CompoundDiscID" integer, intermediate integer, bonus integer, extra integer, exam integer, examcnt integer)
LANGUAGE plpgsql
AS $function$
BEGIN

  DROP TABLE IF EXISTS vRoadMap;
  CREATE TEMPORARY TABLE vRoadMap AS (
    SELECT  view_roadmap.disciplineid,
      view_roadmap.ModuleType,
      view_roadmap.SubmoduleID
    from view_roadmap where view_roadmap.disciplineid in (SELECT view_disciplines_recordbooks.DisciplineID
                                                          FROM view_disciplines_recordbooks
                                                            INNER JOIN students_groups ON students_groups.RecordBookID = view_disciplines_recordbooks.RecordBookID AND
                                                                                          students_groups.SemesterID = view_disciplines_recordbooks.semesterid
                                                          WHERE students_groups.GroupID = pGroupID and students_groups.semesterid = pSemesterID and
                                                                students_groups.state <= 'outlet'
                                                          GROUP BY view_disciplines_recordbooks.DisciplineID)
  );

  return query
  SELECT  vRates.StudentID as "StudentID",
          vRates.DisciplineID as "DisciplineID",
          disciplines.CompoundDiscID as "CompoundDiscID",
          vRates.RateRegular AS intermediate,
          vRates.RateBonus AS bonus,
          vRates.RateExtra AS extra,
          vRates.RateExam AS exam,
          vRates.CntExam AS examCnt
  FROM (
         SELECT  tStudents.StudentID,
           vRoadMap.DisciplineID,
           cast(SUM(CASE WHEN vRoadMap.ModuleType = 'regular' THEN  rt.Rate ELSE  0 END) as integer)AS RateRegular,
           cast(SUM(CASE WHEN vRoadMap.ModuleType = 'extra' THEN  rt.Rate ELSE  0 END)as integer) AS RateExtra,
           cast(SUM(CASE WHEN vRoadMap.ModuleType = 'bonus' THEN  rt.Rate ELSE  0 END)as integer) AS RateBonus,
           MAX(CASE WHEN vRoadMap.ModuleType = 'exam' THEN  rt.Rate ELSE  0 END) AS RateExam,
           cast( SUM(CASE WHEN vRoadMap.ModuleType = 'exam' THEN  1 ELSE  0 END)as integer) AS CntExam
         FROM (select vs.recordbookid, vs.studentid from view_students as vs where vs.groupid=pgroupid and vs.semesterid=psemesterid) tStudents
           CROSS join vRoadMap
           LEFT JOIN rating_table as rt  ON  rt.RecordBookID = tStudents.RecordBookID AND
                                             rt.SubmoduleID = vRoadMap.SubmoduleID
         WHERE rt.Rate IS NOT NULL
         GROUP BY tStudents.StudentID, vRoadMap.DisciplineID
       ) vRates
    INNER JOIN students ON students.ID = vRates.StudentID
    INNER JOIN accounts ON students.AccountID = accounts.ID
    INNER JOIN disciplines ON disciplines.id = vRates.DisciplineID
  ORDER BY    CONCAT(accounts.LastName, accounts.FirstName,  coalesce(accounts.SecondName,'')) ASC,
    vRates.DisciplineID ASC;
END
$function$;

CREATE OR REPLACE FUNCTION public.getratesforgroup(pdisciplineid integer, pgroupid integer)
  RETURNS TABLE("ID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, intermediate integer, bonus integer, exam integer)
LANGUAGE plpgsql
AS $function$

-- DECLARE vInGeneralGroup BOOLEAN DEFAULT FALSE;
declare vSemesterID INT DEFAULT -1;
BEGIN
  vSemesterID = GetDisciplineProperty(pDisciplineID, 'semester');

  /*
       vInGeneralGroup = EXISTS(
      SELECT * FROM disciplines_groups
      WHERE disciplines_groups.DisciplineID = pDisciplineID AND
            disciplines_groups.GroupID = pGroupID
      LIMIT 1
  );
    */


  return query
  SELECT  students.ID			as "ID",
          accounts.LastName	as "LastName",
          accounts.FirstName	as "FirstName",
          accounts.SecondName	as "SecondName",
          vRates.RateRegular 	AS intermediate,
          vRates.RateBonus 	AS bonus,
          vRates.RateExam 	AS exam
  FROM (
         select students_groups.recordbookid,
           cast(SUM(CASE WHEN vRoadMap.Type = 'regular' THEN  rt.Rate ELSE  0 end) as integer) AS RateRegular,
           cast(SUM(CASE WHEN vRoadMap.Type = 'extra' THEN  rt.Rate ELSE  0 END) as integer) AS RateExtra,
           cast(SUM(CASE WHEN vRoadMap.Type = 'bonus' THEN  rt.Rate ELSE  0 END) as integer) AS RateBonus,
           MAX(CASE WHEN vRoadMap.Type = 'exam' THEN  rt.Rate ELSE  0 END) AS RateExam
         FROM students_groups
           CROSS JOIN (SELECT view_roadmap.SubmoduleID, view_roadmap.ModuleType AS Type
                       FROM view_roadmap
                       WHERE view_roadmap.DisciplineID = pDisciplineID
                      ) vRoadMap
           LEFT JOIN rating_table as rt  ON rt.RecordBookID=students_groups.recordbookid and
                                            rt.SubmoduleID = vRoadMap.SubmoduleID
         where students_groups.SemesterID = vSemesterID and students_groups.GroupID = pGroupID and students_groups.State <= 'outlet'
         group by students_groups.recordbookid
       ) vRates
    inner join record_books on record_books.id=vRates.recordbookid
    INNER JOIN students ON students.ID = record_books.studentid
    LEFT JOIN accounts ON students.AccountID = accounts.ID
  ORDER BY CONCAT(accounts.LastName, accounts.FirstName,  coalesce (accounts.SecondName,'')) ASC,
    students.ID ASC;


END
$function$;

DROP FUNCTION if exists public.getgroupsfordisciplineall(pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.getgroupsfordisciplineall(pdisciplineid integer)
  RETURNS TABLE("ID" integer, "GroupNum" integer, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "SpecID" integer, "SpecName" character varying, "SpecAbbr" character varying, "FormID" character varying)
LANGUAGE plpgsql
AS $function$

DECLARE vSemesterID INT DEFAULT -1; vYear INT DEFAULT -1;
BEGIN
  vSemesterID := GetDisciplineProperty(pDisciplineID, 'semester');
  SELECT "year"  into vYear from semesters where semesters.ID = vSemesterID;

  return query
  SELECT  view_groups.GroupID AS "ID",
          view_groups.GroupNum as "GroupNum",
          view_groups.GradeID	as "GradeID",
          view_groups.GradeNum as "GradeNum",
          view_groups."degree" as "Degree",
          view_groups.SpecID as "SpecID",
          view_groups.SpecName as "SpecName",
          view_groups.SpecAbbr as "SpecAbbr",
    (SELECT study_form.FormName from study_form WHERE study_form.id = view_groups.FormID)
  --view_groups.FormID as "FormID"
  FROM (SELECT students_groups.GroupID
        FROM view_disciplines_recordbooks
          INNER JOIN students_groups ON students_groups.RecordBookID = view_disciplines_recordbooks.RecordBookID AND
                                        students_groups.SemesterID = vSemesterID AND
                                        students_groups.State <= 'outlet'
        WHERE view_disciplines_recordbooks.DisciplineID = pDisciplineID AND
              'attach' = COALESCE(view_disciplines_recordbooks.Type, 'attach')
        GROUP BY students_groups.GroupID) tGroup
    INNER JOIN view_groups ON tGroup.GroupID = view_groups.GroupID AND view_groups.Year = vYear
  ORDER BY view_groups.GradeID ASC, view_groups.GroupID ASC;
END;
$function$;

CREATE OR REPLACE FUNCTION public.getdisciplinesforgroup(pgroupid integer, psemesterid integer)
  RETURNS TABLE("ID" integer, "SubjectName" character varying, "Subtype" scientific_disciplinary_coursework, "Type" exam_credit_grading_credit, "CompoundDiscID" integer, "CompoundDiscName" character varying, "CurRate" integer, "MaxRate" integer)
LANGUAGE plpgsql
AS $function$
BEGIN
  return query
  SELECT DISTINCT view_disciplines.DisciplineID AS "ID",
                  view_disciplines.SubjectName as "SubjectName",
                  view_disciplines.Subtype as "Subtype",
                  view_disciplines.ExamType  AS "Type",
                  view_disciplines.CompoundDiscID as "CompoundDiscID",
                  view_disciplines.CompoundDiscName as "CompoundDiscName",
                  view_disciplines.CurRate as "CurRate",
                  view_disciplines.MaxRate as "MaxRate"
  FROM students_groups
    LEFT JOIN view_disciplines_recordbooks ON students_groups.RecordBookID = view_disciplines_recordbooks.RecordBookID
    INNER JOIN view_disciplines ON view_disciplines_recordbooks.DisciplineID = view_disciplines.DisciplineID
  WHERE   students_groups.GroupID = pGroupID AND
          students_groups.SemesterID = pSemesterID AND
          view_disciplines.SemesterID = pSemesterID AND
          students_groups.State <= 'outlet' AND
          -- view_disciplines_recordbooks.Type may be attach or NULL
          COALESCE(view_disciplines_recordbooks.Type, 'attach') = 'attach'
  ORDER BY view_disciplines.ExamType DESC,
    view_disciplines.SubjectName ASC;
end;
$function$;

CREATE OR REPLACE FUNCTION public.discipline_getstudents(pdisciplineid integer)
  RETURNS TABLE("ID" integer, "RecordBookID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "GroupID" integer, "GroupNum" integer, "AttachType" attach_detach)
LANGUAGE plpgsql
AS $function$
DECLARE vSemesterID int DEFAULT -1;
BEGIN
  --select  * from grade.Discipline_GetStudents(
  --	:pDisciplineID 	-- put the pDisciplineID parameter value instead of 'pDisciplineID' (INT)
  --);
  vSemesterID = GetDisciplineProperty(pDisciplineID, 'semester');
  return query
  SELECT  view_students.StudentID AS "ID",
          view_students.RecordBookID as "RecordBookID",
          view_students.LastName as "LastName",
          view_students.FirstName as "FirstName",
          view_students.SecondName as "SecondName",
          view_students.GradeID as "GradeID",
          view_students.GradeNum as "GradeNum",
          view_students."degree" as "Degree",
          view_students.GroupID as "GroupID",
          view_students.GroupNum as "GroupNum",
          view_disciplines_recordbooks."type" AS "AttachType"
  FROM view_disciplines_recordbooks
    INNER JOIN view_students ON  view_disciplines_recordbooks.RecordBookID = view_students.RecordBookID
  WHERE view_students.SemesterID = vSemesterID AND view_disciplines_recordbooks.DisciplineID = pDisciplineID
        and view_students.recordbookid in (select recordbookid from students_groups where state <= 'outlet' and semesterid=vsemesterid)
  ORDER BY    view_disciplines_recordbooks."type" = 'attach' ASC,
    view_students.GradeID ASC,
    view_students.GroupNum ASC,
    view_students.LastName ASC,
    view_students.FirstName ASC;
END
$function$;

ALTER TABLE public.study_groups ADD COLUMN specid integer DEFAULT null;

ALTER TABLE public.study_groups
  DROP CONSTRAINT study_groups_facultyid_gradeid_groupnum_formid_key;

ALTER TABLE ONLY public.study_groups
  ADD CONSTRAINT study_groups_facultyid_gradeid_groupnum_formid_spec_key UNIQUE (facultyid, gradeid, groupnum, formid, specid);

ALTER TABLE ONLY public.study_groups
  ADD CONSTRAINT study_groups_ibfk_5 FOREIGN KEY (specid) REFERENCES specializations(id);

-- script to fill study_groups.specid from latest record in groups_years
-- after creating empty specid column
update study_groups set specid=(
  select groups_years.specializationid from groups_years
  where groups_years.groupid=study_groups.id
  order by groups_years.year desc
  limit 1
) where specid is null;

DROP FUNCTION IF EXISTS public.creategroup(pgradeid integer, pgroupnum integer, pspecname character varying, pfacultyid integer, pyear integer, pformid integer);
CREATE OR REPLACE FUNCTION public.creategroup(pgradeid integer, pgroupnum integer, pspecname character varying, pfacultyid integer, pyear integer, pformid integer)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
DECLARE vGroupID INT DEFAULT null;
        vSpecId INT DEFAULT null;
        vGroupYear INT DEFAULT null;
        vIsSpecMatch BOOL DEFAULT null;

BEGIN
  select specializations.id into vSpecId
  from specializations
  where specializations.facultyid=pFacultyID and specializations."name" like pSpecName;
  if (vSpecId is null) then
    INSERT INTO specializations (Name, Abbr, FacultyID)
    VALUES  (pSpecName, NULL, pFacultyID)
    returning id into vSpecId;
  end if;

  -- create group  vGroupID - UNIQUE (pGradeID, pGroupNum, pFacultyID)
  select study_groups.id into vGroupID
  from study_groups
  where study_groups.gradeid=pgradeID
        and study_groups.groupnum= pGroupNum
        and study_groups.facultyid=pFacultyID
        and study_groups.formid=pFormID
        and study_groups.specid=vSpecId;
  if ( vGroupID  is null) then
    INSERT INTO study_groups (GradeID, GroupNum, FacultyID, FormID, SpecID)
    VALUES (pGradeID, pGroupNum, pFacultyID, pFormID, vSpecId)
    returning id into  vGroupID ;
  end if;


  SELECT groups_years.groupid, groups_years.specializationid = vSpecId
  INTO vGroupYear, vIsSpecMatch
  FROM groups_years
  WHERE groups_years.groupid = vGroupID AND groups_years."year" = pYear
  LIMIT 1;

  IF  (vGroupYear is null) THEN
    INSERT INTO groups_years (GroupID, Year, SpecializationID)
    VALUES (vGroupID, pYear, vSpecId);
  ELSEIF NOT vIsSpecMatch THEN
    RETURN -1;
  END IF;

  RETURN vGroupID;
  EXCEPTION
  when others then
    RETURN -1;
END
$function$;

-- здесь добавлена сортировка занятий по дате, чтобы в таблице они сортировались слева направо по возрастанию
-- их даты проведения
DROP FUNCTION IF EXISTS public.get_lessons_for_discipline(pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.get_lessons_for_discipline(pdisciplineid integer)
  RETURNS TABLE("ID" integer, "LessonDate" date, "LessonType" integer, "LessonName" character varying)
LANGUAGE sql
AS $function$
SELECT lessons.id AS "ID",
       lessons.lesson_date AS "LessonDate",
       lessons.lesson_type AS "LessonType",
       lessons.lesson_name AS "LessonName"
FROM lessons
WHERE lessons.discipline_id = pdisciplineid
      AND lessons.lesson_deleted = 0
ORDER BY lessons.lesson_date;
$function$;

DROP FUNCTION IF EXISTS public.finalreport_getbysemesteryeardisciplinegroup(pyear integer, pnum integer, pdisciplineexternalid character varying, pgroupid integer);
CREATE OR REPLACE FUNCTION public.finalreport_getbysemesteryeardisciplinegroup(pyear integer, pnum integer, pdisciplineexternalid character varying, pgroupid integer)
  RETURNS TABLE("SemesterYear" integer, "SemesterNum" integer, "PlanExternalID" integer,
                "SubjectExternalID" character varying, "RecordBookExternalID" character varying, "ExamType" exam_credit_grading_credit,
                "SemesterRate" bigint, "ExamRate" integer, "Exam2Rate" integer, "Exam3Rate" integer,
                "ExtraRate" integer, "Extra2Rate" integer, "BonusRate" integer)
LANGUAGE plpgsql
AS $function$
declare vSemesterId integer default -1;
BEGIN

  select semesters.id into vSemesterId
  from semesters
  where semesters.year = pyear and semesters.num = pnum;

  return query
  select DisciplineGrades.SemesterYear as "SemesterYear",
         DisciplineGrades.SemesterNum as "SemesterNum",
         DisciplineGrades.PlanExternalID as "PlanExternalDI",
         DisciplineGrades.SubjectExternalID as "SubjectExternalID",
         DisciplineGrades.RecordBookExternalID as "RecordBookExternalID",
         DisciplineGrades.ExamType as "ExamType",
         DisciplineGrades.SemesterRate as "SemesterRate",
         DisciplineGrades.ExamRate as "ExamRate",
         DisciplineGrades.Exam2Rate as "Exam2Rate",
         DisciplineGrades.Exam3Rate as "Exam3Rate",
         DisciplineGrades.ExtraRate as "ExtraRate",
         DisciplineGrades.Extra2Rate as "Extra2Rate",
         DisciplineGrades.BonusRate as "BonusRate"
  from
    (SELECT *
     FROM view_final_report
     WHERE view_final_report.SemesterYear = pyear and
           view_final_report.SemesterNum = pnum and
           view_final_report.SubjectExternalID = pdisciplineexternalid) as DisciplineGrades
    join record_books on record_books.externalid = DisciplineGrades.recordbookexternalid
    join students_groups on students_groups.recordbookid = record_books.id
  where students_groups.semesterid = vSemesterId and students_groups.groupid = pgroupid;
END
$function$;

CREATE OR REPLACE VIEW public.view_final_report
  AS SELECT DISTINCT semesters.year AS semesteryear,
                     semesters.num AS semesternum,
                     study_plans.externalid AS planexternalid,
                     subjects.externalid AS subjectexternalid,
                     record_books.externalid AS recordbookexternalid,
       disciplines.examtype,
                     ( SELECT sum(rating_table.rate) AS sum
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'regular'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       GROUP BY modules.disciplineid, rating_table.recordbookid) AS semesterrate,
                     ( SELECT rating_table.rate
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'exam'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       ORDER BY submodules.ordernum
                       LIMIT 1) AS examrate,
                     ( SELECT rating_table.rate
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'exam'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       ORDER BY submodules.ordernum
                       OFFSET 1
                       LIMIT 1) AS exam2rate,
                     ( SELECT rating_table.rate
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'exam'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       ORDER BY submodules.ordernum
                       OFFSET 2
                       LIMIT 1) AS exam3rate,
                     ( SELECT rating_table.rate
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'extra'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       ORDER BY submodules.ordernum
                       LIMIT 1) AS extrarate,
                     ( SELECT rating_table.rate
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'extra'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       ORDER BY submodules.ordernum
                       OFFSET 1
                       LIMIT 1) AS extra2rate,
                     ( SELECT rating_table.rate
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'bonus'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       LIMIT 1) AS bonusrate
     FROM semesters
       JOIN disciplines ON disciplines.semesterid = semesters.id
       JOIN disciplines_study_plans ON disciplines_study_plans.disciplineid = disciplines.id
       JOIN study_plans ON study_plans.id = disciplines_study_plans.studyplanid
       JOIN subjects ON subjects.id = disciplines.subjectid
       JOIN view_disciplines_recordbooks ON view_disciplines_recordbooks.disciplineid = disciplines.id AND 'attach'::attach_detach = COALESCE(view_disciplines_recordbooks.type, 'attach'::attach_detach)
       JOIN record_books ON record_books.id = view_disciplines_recordbooks.recordbookid
       JOIN record_books_plans ON record_books.id = record_books_plans.recordbookid AND record_books_plans.studyplanid = study_plans.id;