DROP FUNCTION IF EXISTS public.getgroups(pgradeid integer, pfacultyid integer, psemesterid integer);
CREATE OR REPLACE FUNCTION public.getgroups(pgradeid integer, pfacultyid integer, psemesterid integer)
 RETURNS TABLE("ID" integer, "GroupNum" integer, "SpecID" integer, "SpecName" character varying, "SpecAbbr" character varying, "FormID" integer, "FormName" character varying)
 LANGUAGE sql
AS $function$
--select * from public.getgroups(
--	:pgradeid,	-- put the pgradeid parameter value instead of 'pgradeid' (int4)
--	:pfacultyid,	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--	:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--);
SELECT  view_groups.GroupID AS "ID",
        view_groups.GroupNum as "GroupNum",
        view_groups.SpecID as "SpecID",
        view_groups.SpecName as "SpecName",
        view_groups.SpecAbbr as "SpecAbbr",
        view_groups.FormID as "FormID",
  (SELECT study_form.FormName from study_form WHERE study_form.id = view_groups.FormID)
FROM view_groups
  INNER JOIN semesters ON semesters.ID = pSemesterID
WHERE   view_groups.GradeID = pGradeID AND
        view_groups.FacultyID = pFacultyID AND
        view_groups.Year = semesters.Year
ORDER BY view_groups.GroupNum ASC;
$function$;

CREATE OR REPLACE FUNCTION public.createauthtoken(paccountid integer, pdescription character varying, prightmask integer)
 RETURNS character varying
 LANGUAGE plpgsql
AS $function$
    DECLARE
    vToken char(40);
BEGIN
        vToken :=gen_random_uuid();
        INSERT INTO auth_tokens(Token, AccountID, Description, Mask) VALUES (vToken, pAccountID, pDescription, pRightMask);
    RETURN vToken;
END
$function$;

DROP FUNCTION IF EXISTS public.getauthtokens(paccountid integer);
CREATE OR REPLACE FUNCTION public.getauthtokens(paccountid integer)
 RETURNS TABLE("AccountID" integer, "Created" timestamp without time zone, "Accessed" timestamp without time zone, "Mask" integer, "Token" character)
 LANGUAGE sql
AS $function$
-- select * from public.getauthtokens(:paccountid 	-- put the paccountid parameter value instead of 'paccountid' (int4));
-- РµСЃР»Рё РІС‹Р·С‹РІР°С‚СЊ select  public.getauthtokens(...); РІС‹РґР°РµС‚ С‚Р°Р±Р»РёС†Сѓ СЃ РѕРґРЅРёРј СЃРѕР»Р±С†РѕРј - РІСЃРµ РґР°РЅРЅС‹Рµ СЃРєР»РµРЅС‹ РІ СЃРїРёСЃРѕРє
    SELECT accountID as "AccountID",
           created as "Created",
           accessed as "Accessed",
           mask as "Mask",
           token as "Token"
    FROM auth_tokens
    INNER JOIN accounts ON accounts.ID = auth_tokens.AccountID
    WHERE
        auth_tokens.AccountID = pAccountID OR
        pAccountID = 0
    ORDER BY auth_tokens.Created DESC;
$function$;

DROP FUNCTION IF EXISTS public.logformexport(paccountid integer, pdisciplineid character varying, pgroupid integer, psemesterid integer, perror integer);
CREATE OR REPLACE FUNCTION public.logformexport(paccountid integer, pdisciplineid character varying, pgroupid integer, psemesterid integer, perror integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
DECLARE vDisciplineID integer;
begin
  select disciplinesForGroup."ID" into vDisciplineID from
    (select D."ID" as "ID",
            D."SubjectName" as "SubjectName",
            subjects.externalid as "DisciplineExternalID"
     from public.getdisciplinesforgroup(pgroupid, psemesterid) as D
       join disciplines on disciplines.id = D."ID"
       join subjects on disciplines.subjectid = subjects.id) as disciplinesForGroup
  WHERE disciplinesForGroup."DisciplineExternalID" = pdisciplineid;

  INSERT INTO logs_form_export (discipline_id, group_id, account_id, semester_id, error) VALUES (vDisciplineID, pgroupid, paccountid, psemesterid, perror);
  RETURN 0;
END
$function$;

DROP function if EXISTS public.subject_update(psubjectid integer, psubjectname character varying);
CREATE OR REPLACE FUNCTION public.subject_update(psubjectid integer, psubjectname character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
DECLARE rowUpd int;
begin
  UPDATE subjects SET name = psubjectname WHERE id = psubjectid;
  get diagnostics rowUpd = ROW_COUNT;
  RETURN rowUpd-1;

END
$function$;

DROP FUNCTION IF EXISTS public.getstudents(pfacultyid integer, pgradeid integer, pgroupid integer, psemesterid integer, pword1 character varying, pword2 character varying, pword3 character varying);
CREATE OR REPLACE FUNCTION public.getstudents(pfacultyid integer, pgradeid integer, pgroupid integer, psemesterid integer, pword1 character varying, pword2 character varying, pword3 character varying)
 RETURNS TABLE("ID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "AccountID" integer, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "GroupID" integer, "GroupNum" integer)
 LANGUAGE plpgsql
AS $function$
DECLARE FullName VARCHAR;
begin
  return query
  SELECT  view_students.StudentID AS "ID",
          view_students.LastName AS "LastName",
          view_students.FirstName AS "FirstName",
          view_students.SecondName AS "SecondName",
          view_students.AccountID AS "AccountID",
          view_students.GradeID AS "GradeID",
          view_students.GradeNum AS "GradeNum",
          view_students.Degree AS "Degree",
          view_students.GroupID AS "GroupID",
          view_students.GroupNum AS "GroupNum"
  FROM view_students
  where
    recordbookid in (select recordbookid from students_groups where semesterid=psemesterid and state<='outlet') and
    CASE WHEN pSemesterID != 0 THEN view_students.SemesterID = pSemesterID ELSE
      TRUE
    END AND
    CASE WHEN pFacultyID != 0 THEN view_students.FacultyID = pFacultyID ELSE TRUE END AND
    CASE WHEN pGradeID != 0 THEN view_students.GradeID = pGradeID ELSE TRUE END AND
    CASE WHEN pGroupID != 0 THEN view_students.GroupID = pGroupID ELSE TRUE END AND
    CASE WHEN pWord1 != '' THEN

      CONCAT(view_students.LastName, ' ', view_students.FirstName, ' ', view_students.SecondName) LIKE CONCAT('%', pWord1, '%') AND
      CONCAT(view_students.LastName, ' ', view_students.FirstName, ' ', view_students.SecondName) LIKE CONCAT('%', pWord2, '%') AND
      CONCAT(view_students.LastName, ' ', view_students.FirstName, ' ', view_students.SecondName) LIKE CONCAT('%', pWord3, '%')
    ELSE TRUE end
  order by "LastName","FirstName","SecondName";
END
$function$;

CREATE OR REPLACE FUNCTION public.faculty_getdeans(pfacultyid integer)
 RETURNS TABLE("FullName" character varying)
 LANGUAGE plpgsql
AS $function$
begin
  return query
  select GetUserFullNameByAccountID(accounts.ID) as "FullName"
  from accounts
  join teachers on teachers.accountid = accounts.id
  where teachers.facultyid = pfacultyid and accounts.userroleid = 4;
END;
$function$;

CREATE OR REPLACE FUNCTION public.isformoutdated(pdisciplineid integer, pgroupid integer, psemesterid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
DECLARE vLastDate timestamp DEFAULT NULL;
        vCountNewGrades integer DEFAULT 0;
begin
  SELECT logs_form_export.date INTO vLastDate
  FROM logs_form_export
  WHERE logs_form_export.discipline_id = pdisciplineid
        AND logs_form_export.group_id = pgroupid
        AND logs_form_export.semester_id = psemesterid
  ORDER BY logs_form_export.date DESC
  LIMIT 1;

  SELECT count(logs_rating.date) INTO vCountNewGrades FROM
    logs_rating JOIN students_groups
      ON logs_rating.recordbookid = students_groups.recordbookid
    JOIN submodules ON logs_rating.submoduleid = submodules.id
    JOIN modules ON submodules.moduleid = modules.id
    JOIN disciplines ON modules.disciplineid = disciplines.id
  WHERE students_groups.semesterid = psemesterid
        AND students_groups.groupid = pgroupid
        AND disciplines.id = pdisciplineid
        AND logs_rating.date > vLastDate;

  return vCountNewGrades;
END
$function$;

