DROP VIEW IF EXISTS public.view_final_report;
CREATE OR REPLACE VIEW public.view_final_report
  AS SELECT DISTINCT semesters.year AS semesteryear,
                     semesters.num AS semesternum,
                     study_plans.externalid AS planexternalid,
                     subjects.externalid AS subjectexternalid,
                     record_books.externalid AS recordbookexternalid,
       disciplines.examtype,
                     ( SELECT sum(rating_table.rate) AS sum
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'regular'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       GROUP BY modules.disciplineid, rating_table.recordbookid) AS semesterrate,
                     ( SELECT  (case
                          when rt.Rate is not null then rt.Rate
                          when rt.Rate is null then
                            case epo."type"
                               when 'pass' then -2
                               when 'absence' then -1
                            END
                          END) as "rate"
                        FROM view_roadmap AS vr
                        LEFT JOIN rating_table AS rt ON  vr.SubmoduleID = rt.SubmoduleID AND rt.RecordBookID = record_books.id
                        LEFT JOIN exam_period_options AS epo ON  epo.SubmoduleID = vr.SubmoduleID AND epo.RecordBookID = record_books.id
                        WHERE   vr.DisciplineID = disciplines.id AND vr.ModuleType = 'exam'::regular_exam_bonus_extra
                        ORDER BY vr.ModuleType,
                              vr.ModuleOrderNum ASC,
                              vr.SubmoduleOrderNum ASC
                        LIMIT 1
                     ) AS examrate,
                     ( SELECT  (case
                          when rt.Rate is not null then rt.Rate
                          when rt.Rate is null then
                            case epo."type"
                               when 'pass' then 0
                               when 'absence' then -1
                            END
                          END) as "rate"
                        FROM view_roadmap AS vr
                        LEFT JOIN rating_table AS rt ON  vr.SubmoduleID = rt.SubmoduleID AND rt.RecordBookID = record_books.id
                        LEFT JOIN exam_period_options AS epo ON  epo.SubmoduleID = vr.SubmoduleID AND epo.RecordBookID = record_books.id
                        WHERE   vr.DisciplineID = disciplines.id AND vr.ModuleType = 'exam'::regular_exam_bonus_extra
                        ORDER BY vr.ModuleType,
                              vr.ModuleOrderNum ASC,
                              vr.SubmoduleOrderNum ASC
                       OFFSET 1
                       LIMIT 1) AS exam2rate,
                     ( SELECT  (case
                          when rt.Rate is not null then rt.Rate
                          when rt.Rate is null then
                            case epo."type"
                               when 'pass' then 0
                               when 'absence' then -1
                            END
                          END) as "rate"
                        FROM view_roadmap AS vr
                        LEFT JOIN rating_table AS rt ON  vr.SubmoduleID = rt.SubmoduleID AND rt.RecordBookID = record_books.id
                        LEFT JOIN exam_period_options AS epo ON  epo.SubmoduleID = vr.SubmoduleID AND epo.RecordBookID = record_books.id
                        WHERE   vr.DisciplineID = disciplines.id AND vr.ModuleType = 'exam'::regular_exam_bonus_extra
                        ORDER BY vr.ModuleType,
                              vr.ModuleOrderNum ASC,
                              vr.SubmoduleOrderNum ASC
                       OFFSET 2
                       LIMIT 1) AS exam3rate,
                     ( SELECT rating_table.rate
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'extra'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       ORDER BY submodules.ordernum
                       LIMIT 1) AS extrarate,
                     ( SELECT rating_table.rate
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'extra'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       ORDER BY submodules.ordernum
                       OFFSET 1
                       LIMIT 1) AS extra2rate,
                     ( SELECT rating_table.rate
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'bonus'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       LIMIT 1) AS bonusrate
     FROM semesters
       JOIN disciplines ON disciplines.semesterid = semesters.id
       JOIN disciplines_study_plans ON disciplines_study_plans.disciplineid = disciplines.id
       JOIN study_plans ON study_plans.id = disciplines_study_plans.studyplanid
       JOIN subjects ON subjects.id = disciplines.subjectid
       JOIN view_disciplines_recordbooks ON view_disciplines_recordbooks.disciplineid = disciplines.id AND 'attach'::attach_detach = COALESCE(view_disciplines_recordbooks.type, 'attach'::attach_detach)
       JOIN record_books ON record_books.id = view_disciplines_recordbooks.recordbookid
       JOIN record_books_plans ON record_books.id = record_books_plans.recordbookid AND record_books_plans.studyplanid = study_plans.id;


CREATE SEQUENCE seq_faq
  START WITH 1
  INCREMENT BY 1
  NO MINVALUE
  NO MAXVALUE
  CACHE 1;

DROP TABLE IF EXISTS faq;
CREATE TABLE faq (
  id integer DEFAULT nextval('seq_faq'::regclass) NOT NULL,
  question CHARACTER VARYING NOT NULL,
  answer CHARACTER VARYING NOT NULL,
  colour VARCHAR(6) DEFAULT 'FFFFFF',
  popularity INTEGER DEFAULT 0 NOT NULL,
  date_added timestamp(0) without time zone DEFAULT now() NOT NULL
);

ALTER TABLE ONLY faq
  ADD CONSTRAINT faq_pkey PRIMARY KEY (id);


CREATE OR REPLACE FUNCTION public.getFAQ()
  RETURNS TABLE(Question CHARACTER VARYING, Answer CHARACTER VARYING, Color VARCHAR(6))
LANGUAGE plpgsql
AS $function$
BEGIN
  return query
  SELECT  faq."question" as "Question",
          faq."answer" as "Answer",
          faq."colour" as "Color"
  FROM faq
  ORDER BY  faq."popularity" DESC;
END
$function$;


CREATE OR REPLACE FUNCTION public.addFAQ(pquestion CHARACTER VARYING, panswer CHARACTER VARYING, pcolor VARCHAR(6))
  RETURNS integer
LANGUAGE plpgsql
AS $function$
begin
  INSERT INTO faq(question, answer, colour) VALUES (pquestion, panswer, pcolor);
  RETURN 0;
END
$function$;

CREATE OR REPLACE FUNCTION public.discipline_setrate(pteacherid integer, precordbookid integer, psubmoduleid integer, prate integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vDisciplineID INT DEFAULT -1;
               vMaxRate INT DEFAULT -1;
               vModuleType regular_exam_bonus_extra;
    DECLARE vIsOver int DEFAULT 0;
               vIsLocked int DEFAULT 0;
               vIsUsed int2 DEFAULT 0;
               vIsUpdated boolean default false;
begin
--    select Discipline_SetRate(
--    :pTeacherID,    -- put the pTeacherID parameter value instead of 'pTeacherID' (INT)
--    :pRecordBookID,    -- put the pRecordBookID parameter value instead of 'pRecordBookID' (INT)
--    :pSubmoduleID,    -- put the pSubmoduleID parameter value instead of 'pSubmoduleID' (INT)
--    :pRate     -- put the pRate parameter value instead of 'pRate' (INT)
--);

      -- 0) check rights здесь еще нельзя проверять, т.к. нет vDisciplineID
      -- нужно переставить очередность
    /*  IF NOT InternalIsStudentAttached(pRecordBookID, vDisciplineID) OR
       NOT InternalIsTeacherBound(pTeacherID, vDisciplineID) THEN
        RETURN 1;
      END IF;*/

	-- http://gitlab.mmcs.sfedu.ru/it-lab/grade/issues/403
	-- получение VdisciplineID переставлено перед проверкой прав и удалением баллов
	vIsOver = 0;
   SELECT  disciplines.ID,
            disciplines.IsLocked,
            disciplines.Milestone,
            submodules.IsUsed,
            submodules.maxRate,
            modules."type"
        INTO vDisciplineID, vIsLocked, vIsOver, vIsUsed, vMaxRate, vModuleType
        FROM submodules
        INNER JOIN modules                ON  submodules.ModuleID = modules.ID
        INNER JOIN disciplines            ON  modules.DisciplineID = disciplines.ID
        WHERE   submodules.ID = pSubmoduleID
        LIMIT 1;

	-- 1) check rights ПОЧЕМУ БЫЛО ТАК ПОЗДНО ???? см п.0
    IF NOT InternalIsStudentAttached(pRecordBookID, vDisciplineID) OR
       NOT InternalIsTeacherBound(pTeacherID, vDisciplineID) THEN
        RETURN 1;
    END IF;

    --2) check, you can't rate regular and bonus after milestone
    IF (vIsOver=1) AND (vModuleType = 'regular' OR vModuleType = 'bonus') THEN --1 - regular, 3 - bonus
        RETURN 2;
    END IF;

        IF pRate < 0 THEN
        INSERT INTO logs_rating (RecordBookID, SubmoduleID, TeacherID, Rate, "action" ) VALUES
            (pRecordBookID, pSubmoduleID, pTeacherID, pRate, 'delete');

        -- TODO: extract method log rate
        DELETE FROM rating_table
            WHERE   rating_table.RecordBookID = pRecordBookID AND
                    rating_table.SubmoduleID = pSubmoduleID
           ;
        RETURN 0;
        END IF;


    -- correct max rate for extra module
    IF vModuleType = 'extra' THEN -- 4 - extra
         vMaxRate = CalculateMaxRateForExtra(vDisciplineID, pRecordBookID);
    END IF;


    --3) check, max rate exceeding
    IF  pRate > vMaxRate THEN
        RETURN 3;
    END IF;

     -- add rate, or update old
  visUpdated = exists (select * from rating_table where RecordBookID=pRecordBookID and  SubmoduleID= pSubmoduleID);

 INSERT INTO rating_table (RecordBookID, TeacherID, SubmoduleID, Rate, "date")
        VALUES  (pRecordBookID, pTeacherID, pSubmoduleID, pRate, current_date )
        on conflict on constraint rating_table_pkey do
         update set
            TeacherID  = pTeacherID, --(@isUpdated := pTeacherID),
            Rate       = pRate,
            "date"       = current_date;
        -- returning true into visUpdated ; ---??????? если не получится, разделить insert и update
          --log rate

    INSERT INTO logs_rating (RecordBookID, SubmoduleID, TeacherID, Rate, "action" )
        VALUES (pRecordBookID, pSubmoduleID, pTeacherID, pRate, cast (iif_sql(visUpdated, 'change', 'add') as add_change_delete) ); -- РјРѕР¶РµС‚ РЅСѓР¶РЅРѕ cast(   as add_change_delete) ???
   -- lock discipline for structure editing

    IF  vIsLocked = 0 THEN
        UPDATE disciplines
            SET IsLocked = 1
            WHERE disciplines.ID = vDisciplineID
            ;
    END IF;

--add submodule to max rate counting (see triggers)
    IF vIsUsed=0 THEN
        UPDATE submodules
            SET IsUsed = 1
            where ID = pSubmoduleID
           ;
    END IF;

    RETURN 0;
EXCEPTION
when others then RETURN -1;
end
 $function$;